@@ -1,6 +1,6 @@
-/*  controls_dialog.h
+/*  $Id: controls_dialog.h,v 1.11 2004/09/13 20:31:39 jasper Exp $
  *  
- *  Copyright (C) 2002 Jasper Huijsmans (huysmans@users.sourceforge.net)
+ *  Copyright (C) 2002,2004 Jasper Huijsmans (jasper@xfce.org)
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -20,9 +20,10 @@
 #ifndef __XFCE_CONTROLS_DIALOG_H__
 #define __XFCE_CONTROLS_DIALOG_H__
 
+#include <gmodule.h>
 #include <panel/global.h>
 
-void controls_dialog (Control * control);
-void destroy_controls_dialog (void);
+G_MODULE_IMPORT void controls_dialog (Control * control);
+G_MODULE_IMPORT void destroy_controls_dialog (void);
 
 #endif /* __XFCE_CONTROLS_DIALOG_H__ */
