@@ -1,4 +1,4 @@
-/*  $Id: panel.c,v 1.116 2004/09/19 09:29:16 jasper Exp $
+/*  $Id: panel.c,v 1.130 2004/11/10 19:21:14 jasper Exp $
  *
  *  Copyright 2002-2004 Jasper Huijsmans (jasper@xfce.org)
  *
@@ -44,6 +44,7 @@
 #define HIDDEN_SIZE	3
 #define HANDLE_WIDTH	10
 #define SNAP_WIDTH	25
+#define OPAQUE          0xffffffff
 
 
 /* typedefs *
@@ -68,6 +69,11 @@ struct _PanelPrivate
 };
 
 
+/* static globals *
+ * -------------- */
+static guint transparency = 0xc0000000;
+
+
 /* exported globals *
  * ---------------- */
 
@@ -115,12 +121,77 @@ static void init_settings (Panel * p);
 /* positioning and position related functions *
  * ------------------------------------------ */
 
+static void
+get_opacity_setting (void)
+{
+    char *file = 
+        xfce_resource_lookup (XFCE_RESOURCE_CONFIG,
+                              "xfce4" G_DIR_SEPARATOR_S "transparency");
+
+    if (file)
+    {
+        FILE *fp;
+        
+        if ((fp = fopen (file, "r")) != NULL)
+        {
+            char line[255];
+            guint trans;
+
+            while (fgets(line, sizeof (line), fp))
+            {
+                if (sscanf (line, "panel = %u", &trans) > 0)
+                {
+                    trans = CLAMP (trans, 0, 80);
+
+                    transparency = OPAQUE - rint ((double)trans * OPAQUE / 100);
+
+                    DBG ("transparency: %u\n", trans);
+                    DBG ("opacity: 0x%x\n", transparency);
+
+                    break;
+                }
+            }
+
+            fclose (fp);
+        }
+        
+        g_free (file);
+    }
+}
+
+static void
+set_opacity (Panel * p, gboolean translucent)
+{
+    static gboolean initialized = FALSE;
+    guint opacity;
+
+    if (G_UNLIKELY (!initialized))
+    {
+        get_opacity_setting ();
+        
+        initialized = TRUE;
+    }
+
+    if (transparency == OPAQUE)
+        return;
+    
+    opacity = (translucent ? transparency : OPAQUE);
+    gdk_error_trap_push ();
+
+    gdk_property_change (p->toplevel->window,
+			 gdk_atom_intern ("_NET_WM_WINDOW_OPACITY", FALSE),
+			 gdk_atom_intern ("CARDINAL", FALSE), 32,
+			 GDK_PROP_MODE_REPLACE, (guchar *) & opacity, 1L);
+
+    gdk_error_trap_pop ();
+}
+
 static void
 update_partial_struts (Panel * p)
 {
     gulong data[12] = { 0, };
 
-    if (!p->priv->settings.autohide && p->priv->settings.layer == 0)
+    if (!p->priv->settings.autohide)
     {
 	int w, h, x, y;
 
@@ -325,16 +396,19 @@ restrict_position (Panel * p, int *x, int *y)
 	p->priv->offset = *y - p->priv->monitor_geometry.y;
     }
 
-    DBG ("++ position: %d, %d\n", p->position.x, p->position.y);
-    DBG ("   monitor: %d\n", p->priv->monitor);
-    DBG ("   side: %s\n",
+    DBG ("\n"
+         " ++ position: %d, %d\n"
+         "    monitor: %d\n"
+         "    side: %s\n"
+         "    state: %s\n",
+         p->position.x, p->position.y,
+         p->priv->monitor,
 	 p->priv->side == LEFT ? "left" :
-	 p->priv->side == RIGHT ? "right" :
-	 p->priv->side == TOP ? "top" : "bottom");
-    DBG ("   state: %s\n",
+            p->priv->side == RIGHT ? "right" :
+                p->priv->side == TOP ? "top" : "bottom",
 	 p->priv->pos_state == XFCE_POS_STATE_CENTER ? "center" :
-	 p->priv->pos_state == XFCE_POS_STATE_START ? "start" :
-	 p->priv->pos_state == XFCE_POS_STATE_END ? "end" : "none");
+             p->priv->pos_state == XFCE_POS_STATE_START ? "start" :
+                 p->priv->pos_state == XFCE_POS_STATE_END ? "end" : "none");
 }
 
 static void
@@ -435,16 +509,19 @@ panel_set_position (Panel * p)
 	}
     }
 
-    DBG (" ++ position: %d, %d\n", p->position.x, p->position.y);
-    DBG ("    monitor: %d\n", p->priv->monitor);
-    DBG ("    side: %s\n",
+    DBG ("\n"
+         " ++ position: %d, %d\n"
+         "    monitor: %d\n"
+         "    side: %s\n"
+         "    state: %s\n",
+         p->position.x, p->position.y,
+         p->priv->monitor,
 	 p->priv->side == LEFT ? "left" :
-	 p->priv->side == RIGHT ? "right" :
-	 p->priv->side == TOP ? "top" : "bottom");
-    DBG ("    state: %s\n",
+            p->priv->side == RIGHT ? "right" :
+                p->priv->side == TOP ? "top" : "bottom",
 	 p->priv->pos_state == XFCE_POS_STATE_CENTER ? "center" :
-	 p->priv->pos_state == XFCE_POS_STATE_START ? "start" :
-	 p->priv->pos_state == XFCE_POS_STATE_END ? "end" : "none");
+             p->priv->pos_state == XFCE_POS_STATE_START ? "start" :
+                 p->priv->pos_state == XFCE_POS_STATE_END ? "end" : "none");
 
     gtk_window_move (GTK_WINDOW (p->toplevel), p->position.x, p->position.y);
 
@@ -823,6 +900,7 @@ panel_unhide_timeout (Panel * p)
     if (p->hidden)
     {
 	panel_set_hidden (p, FALSE);
+        gtk_widget_queue_resize (p->toplevel);
     }
 
     return FALSE;
@@ -831,23 +909,25 @@ panel_unhide_timeout (Panel * p)
 static gboolean
 panel_enter (GtkWindow * w, GdkEventCrossing * event, Panel * p)
 {
-    if (!(settings.autohide))
-	return FALSE;
-
     if (event->detail != GDK_NOTIFY_INFERIOR)
     {
-	if (p->hide_timeout)
-	{
-	    g_source_remove (p->hide_timeout);
-	    p->hide_timeout = 0;
-	}
+        set_opacity (p, FALSE);
 
-	if (!p->unhide_timeout)
-	{
-	    p->unhide_timeout =
-		g_timeout_add (UNHIDE_TIMEOUT,
-			       (GSourceFunc) panel_unhide_timeout, p);
-	}
+        if (settings.autohide)
+        {
+            if (p->hide_timeout)
+            {
+                g_source_remove (p->hide_timeout);
+                p->hide_timeout = 0;
+            }
+
+            if (!p->unhide_timeout)
+            {
+                p->unhide_timeout =
+                    g_timeout_add (UNHIDE_TIMEOUT,
+                                   (GSourceFunc) panel_unhide_timeout, p);
+            }
+        }
     }
 
     return FALSE;
@@ -856,23 +936,25 @@ panel_enter (GtkWindow * w, GdkEventCrossing * event, Panel * p)
 static gboolean
 panel_leave (GtkWidget * w, GdkEventCrossing * event, Panel * p)
 {
-    if (!(settings.autohide))
-	return FALSE;
-
     if (event->detail != GDK_NOTIFY_INFERIOR)
     {
-	if (p->unhide_timeout)
-	{
-	    g_source_remove (p->unhide_timeout);
-	    p->unhide_timeout = 0;
-	}
+        set_opacity (p, TRUE);
 
-	if (!p->hide_timeout)
-	{
-	    p->hide_timeout =
-		g_timeout_add (HIDE_TIMEOUT, (GSourceFunc) panel_hide_timeout,
-			       p);
-	}
+        if (settings.autohide)
+        {
+            if (p->unhide_timeout)
+            {
+                g_source_remove (p->unhide_timeout);
+                p->unhide_timeout = 0;
+            }
+
+            if (!p->hide_timeout)
+            {
+                p->hide_timeout =
+                    g_timeout_add (HIDE_TIMEOUT, 
+                                   (GSourceFunc) panel_hide_timeout, p);
+            }
+        }
     }
 
     return FALSE;
@@ -928,9 +1010,25 @@ panel_allocate_cb (GtkWidget * window, GtkAllocation * allocation, Panel * p)
 	|| allocation->height != p->priv->req.height)
     {
 	panel_set_position (p);
+        gtk_widget_queue_draw (p->toplevel);
     }
 }
 
+static void
+set_panel_window_properties (Panel * p)
+{
+    GtkWindow *window = GTK_WINDOW (p->toplevel);
+    
+    gtk_window_set_type_hint (window, GDK_WINDOW_TYPE_HINT_DOCK);
+
+    gtk_window_set_decorated (window, FALSE);
+    gtk_window_set_resizable (window, FALSE);
+    gtk_window_stick (window);
+
+    gtk_window_set_skip_taskbar_hint (window, TRUE);
+    gtk_window_set_skip_pager_hint (window, TRUE);
+}
+
 static GtkWidget *
 create_panel_window (Panel * p)
 {
@@ -942,14 +1040,9 @@ create_panel_window (Panel * p)
     window = GTK_WINDOW (w);
 
     gtk_window_set_title (window, _("Xfce Panel"));
-    gtk_window_set_decorated (window, FALSE);
-    gtk_window_set_resizable (window, FALSE);
-    gtk_window_stick (window);
-
-    gtk_window_set_skip_taskbar_hint (window, TRUE);
-    gtk_window_set_skip_pager_hint (window, TRUE);
-
     gtk_window_set_gravity (window, GDK_GRAVITY_STATIC);
+    gtk_window_set_type_hint (window, GDK_WINDOW_TYPE_HINT_DOCK);
+    gtk_window_stick (window);
 
     pb = get_panel_pixbuf ();
     gtk_window_set_icon (window, pb);
@@ -1078,6 +1171,8 @@ create_panel (void)
 
     p->priv->is_created = TRUE;
 
+    set_panel_window_properties (p);
+    
     panel_set_position (p);
 
     DBG (" ++ position: %d, %d\n", panel.position.x, panel.position.y);
@@ -1100,6 +1195,8 @@ create_panel (void)
 
     if (p->priv->settings.autohide)
 	panel_set_autohide (TRUE);
+    
+    set_opacity (p, TRUE);
 
     update_partial_struts (p);
 
@@ -1188,18 +1285,20 @@ panel_set_orientation (int orientation)
     else
 	panel_center (RIGHT);
 
+    set_panel_window_properties (&panel);
+
     gtk_widget_show_now (panel.toplevel);
 
     /* size sometimes changes after showing */
     panel_set_position (&panel);
 
-    gtk_window_stick (GTK_WINDOW (panel.toplevel));
-
     panel_set_layer (panel.priv->settings.layer);
 
     if (hidden)
 	panel_set_autohide (TRUE);
 
+    set_opacity (&panel, TRUE);
+
     update_partial_struts (&panel);
 }
 
@@ -1207,6 +1306,7 @@ G_MODULE_EXPORT /* EXPORT:panel_set_layer */
 void
 panel_set_layer (int layer)
 {
+#if 0
     panel.priv->settings.layer = layer;
 
     /* backwards compat */
@@ -1250,6 +1350,7 @@ panel_set_layer (int layer)
 
 #endif
     }
+#endif
 }
 
 G_MODULE_EXPORT /* EXPORT:panel_set_size */
@@ -1352,13 +1453,16 @@ panel_set_autohide (gboolean hide)
     {
 	DBG ("add hide timeout");
 
-	panel.hide_timeout = g_timeout_add (1000,
-					    (GSourceFunc) panel_hide_timeout,
-					    &panel);
+        if (!panel.hide_timeout)
+        {
+            panel.hide_timeout = 
+                g_timeout_add (1000, (GSourceFunc) panel_hide_timeout, &panel);
+        }
     }
     else
     {
-	panel_set_hidden (&panel, hide);
+	panel_set_hidden (&panel, FALSE);
+        gtk_widget_queue_resize (panel.toplevel);
     }
 
     update_partial_struts (&panel);
@@ -1403,6 +1507,7 @@ panel_parse_xml (xmlNodePtr node)
 	g_free (value);
     }
 
+#if 0
     value = xmlGetProp (node, (const xmlChar *) "layer");
 
     if (value)
@@ -1413,6 +1518,7 @@ panel_parse_xml (xmlNodePtr node)
 
 	g_free (value);
     }
+#endif
 
     value = xmlGetProp (node, (const xmlChar *) "size");
 
@@ -1607,9 +1713,11 @@ panel_write_xml (xmlNodePtr root)
     snprintf (value, 2, "%d", settings.orientation);
     xmlSetProp (node, "orientation", value);
 
+#if 0
     snprintf (value, 2, "%d", settings.layer);
     xmlSetProp (node, "layer", value);
-
+#endif
+    
     snprintf (value, 2, "%d", settings.size);
     xmlSetProp (node, "size", value);
 
@@ -1648,17 +1756,24 @@ panel_write_xml (xmlNodePtr root)
 static void
 menu_destroyed (GtkWidget * menu, Panel * p)
 {
-    if (p->priv->settings.autohide
-	&& gdk_window_at_pointer (NULL, NULL) != p->toplevel->window)
+    panel_unblock_autohide (p);
+
+    if (p->priv->block_autohide == 0 
+        && gdk_window_at_pointer (NULL, NULL) != p->toplevel->window)
     {
-	GdkEvent *ev = gdk_event_new (GDK_LEAVE_NOTIFY);
+        if (p->priv->settings.autohide)
+        {
+            GdkEvent *ev = gdk_event_new (GDK_LEAVE_NOTIFY);
 
-	((GdkEventCrossing *) ev)->time = GDK_CURRENT_TIME;
-	((GdkEventCrossing *) ev)->detail = GDK_NOTIFY_NONLINEAR;
+            ((GdkEventCrossing *) ev)->time = GDK_CURRENT_TIME;
+            ((GdkEventCrossing *) ev)->detail = GDK_NOTIFY_NONLINEAR;
 
-	gtk_widget_event (p->toplevel, ev);
+            gtk_widget_event (p->toplevel, ev);
 
-	gdk_event_free (ev);
+            gdk_event_free (ev);
+        }
+        
+        set_opacity (p, TRUE);
     }
 }
 
@@ -1667,7 +1782,8 @@ void
 panel_register_open_menu (GtkWidget * menu)
 {
     g_return_if_fail (GTK_IS_WIDGET (menu));
-
+ 
+    panel_block_autohide (&panel);
     g_signal_connect (menu, "deactivate", G_CALLBACK (menu_destroyed),
 		      &panel);
 }
