@@ -1,4 +1,4 @@
-/*  $Id: mailcheck.c,v 1.48 2004/09/19 09:29:16 jasper Exp $
+/*  $Id: mailcheck.c,v 1.50 2004/11/11 09:21:24 jasper Exp $
  *
  *  Copyright (C) 2002-2004 Jasper Huijsmans (jasper@xfce.org)
  *
@@ -116,6 +116,47 @@ get_mail_pixbuf (void)
     return pb;
 }
 
+static void
+_dimm_icon (GdkPixbuf * pixbuf)
+{
+    int x, y, pixel_stride, row_stride;
+    guchar *row, *pixels;
+    int w, h;
+
+    if (!pixbuf)
+        return;
+        
+    if (!gdk_pixbuf_get_has_alpha (pixbuf))
+    {
+        gdk_pixbuf_saturate_and_pixelate (pixbuf, pixbuf, 0, TRUE);
+        return;
+    }
+
+    w = gdk_pixbuf_get_width (pixbuf);
+    h = gdk_pixbuf_get_height (pixbuf);
+
+    pixel_stride = 4;
+
+    row = gdk_pixbuf_get_pixels (pixbuf);
+    row_stride = gdk_pixbuf_get_rowstride (pixbuf);
+
+    for (y = 0; y < h; y++)
+    {
+        pixels = row;
+
+        for (x = 0; x < w; x++)
+        {
+            pixels[3] /= 2;
+
+            pixels += pixel_stride;
+        }
+
+        row += row_stride;
+    }
+
+    gdk_pixbuf_saturate_and_pixelate (pixbuf, pixbuf, 0, TRUE);
+}
+
 static void
 reset_mailcheck_icons (t_mailcheck * mc)
 {
@@ -133,7 +174,7 @@ reset_mailcheck_icons (t_mailcheck * mc)
     g_return_if_fail (mc->newmail_pb != NULL);
 
     mc->nomail_pb = gdk_pixbuf_copy (mc->newmail_pb);
-    gdk_pixbuf_saturate_and_pixelate (mc->nomail_pb, mc->nomail_pb, 0, TRUE);
+    _dimm_icon (mc->nomail_pb);
 
     mc->oldmail_pb = mc->nomail_pb;
     g_object_ref (mc->oldmail_pb);
@@ -436,6 +477,19 @@ run_mailcheck (t_mailcheck * mc)
     }
 }
 
+/* set mailbox type */
+static void
+set_mbox_type(t_mailcheck *mc)
+{
+    if (strncmp (mc->mbox, "pop3://", 7 * sizeof (char)) == 0)
+    {
+        mc->pop3 = TRUE;
+        sscanf (mc->mbox, "pop3://%[^:]:%[^@]@%s",
+		mc->pop3_username,
+	    	mc->pop3_password, mc->pop3_hostname);
+    }
+} 
+
 static void
 mailcheck_read_config (Control * control, xmlNodePtr node)
 {
@@ -473,14 +527,8 @@ mailcheck_read_config (Control * control, xmlNodePtr node)
 	    {
 		g_free (mc->mbox);
 		mc->mbox = (char *) value;
-
-		if (strncmp (mc->mbox, "pop3://", 7 * sizeof (char)) == 0)
-		{
-		    mc->pop3 = TRUE;
-		    sscanf (mc->mbox, "pop3://%[^:]:%[^@]@%s",
-			    mc->pop3_username,
-			    mc->pop3_password, mc->pop3_hostname);
-		}
+		
+		set_mbox_type(mc);
 	    }
 	}
 	else if (xmlStrEqual
@@ -708,6 +756,7 @@ mailcheck_apply_options (MailDialog * md)
     {
 	g_free (mc->mbox);
 	mc->mbox = g_strdup (tmp);
+	set_mbox_type(mc);
     }
 
     tmp = gtk_entry_get_text (GTK_ENTRY (md->newmail_cmd_entry));
